/**
 * mzl-pull-refresh.js
 * @author mzl
 *                         /$$
 *                        | $$
 * /$$$$$$/$$$$  /$$$$$$$$| $$
 *| $$_  $$_  $$|____ /$$/| $$
 *| $$ \ $$ \ $$   /$$$$/ | $$
 *| $$ | $$ | $$  /$$__/  | $$
 *| $$ | $$ | $$ /$$$$$$$$| $$
 *|__/ |__/ |__/|________/|__/
 * 
 */
(function(window) {
    'use strict';
    /**
     * Extend obj function
     *
     * This is an object extender function. It allows us to extend an object
     * by passing in additional variables and overwriting the defaults.
     * 总高度:300
     * 图片高度:70,marginTop:200
     * 文字高度:30
     */
    var PullToRefresh = function (params) {
        this._init(params);
    }
    PullToRefresh.prototype.options = {
        //滑动容器
        container: document.querySelector('#mzl-load-container'),
        loadingWrap:"",
        imageEl: "",
        titleEl: "",
        textDown : '下拉刷新',
        textUp : '松开刷新',
        textRefresh : '刷新中...',
        textSuccess : '刷新成功',
        textError : '刷新失败',
        pullImage: "../image/pull_1.png", //下拉时图片
        transformImage:"../image/pull_2.png", //转换时图片
        loadingImage:"../image/pull.gif", //加载中图片
        callback: false,
        friction: 2.5,
        triggerDistance: 180//达到此高度可以刷新
    };

    /**
     * Pan event parameters
     * @type {Object}
     */
    PullToRefresh.prototype.touch = {
        loading: false
    };
    
    /**
     * can pull
     * @type bool
     */
    PullToRefresh.prototype.canPull = true;

    var touchYDelta;
    var docElem = window.document.documentElement,
        loadWrapH,
        win = {width: window.innerWidth, height: window.innerHeight},
        winfactor= 0.2,
        translateVal,
        firstTouchY, initialScroll;
    var WrapTop = 300;
    PullToRefresh.prototype._init = function(params) {
        extend(this.options, params);
        if(this.options.container){
            this.options.container.addEventListener('touchstart', this.touchStart.bind(this));
            this.options.container.addEventListener('touchmove', this.touchMove.bind(this));
            this.options.container.addEventListener('touchend', this.touchEnd.bind(this));
            this.createLoadWrap();
        }else{
            console.error("PullToRefresh no container");
        }
    };
    PullToRefresh.prototype.createLoadWrap = function(){
        this.options.container.style.webkitTransform =
        this.options.container.style.transform = 'translateY('+(-WrapTop)+'px)';
        this.options.container.style.position = 'absolute';
        this.options.container.style.width = '100%';
        
        // 创建load区域
        if(!this.options.loadingWrap){
            this.options.loadingWrap = document.createElement("div");
            this.options.loadingWrap.style.textAlign = "center";
            this.options.loadingWrap.style.backgroundColor = "#f1f1f1";
            this.options.container.insertBefore(this.options.loadingWrap,this.options.container.firstChild);
        }
        // 获取load高度
        this.loadWrapH = this.options.loadingWrap.offsetHeight;
        // load图片
        this.options.imageEl = document.createElement('img');
        this.options.imageEl.setAttribute("src",this.options.pullImage);
        this.options.imageEl.style.height = 100*0.7+"px";
        this.options.imageEl.style.marginTop = WrapTop/1.5+"px";
        this.options.imageEl.style.verticalAlign = "middle";
        this.options.loadingWrap.appendChild(this.options.imageEl);
        // load标题
        this.options.titleEl = document.createElement('div');
        this.options.titleEl.textContent = this.options.textDown;
        this.options.titleEl.style.height = 100*(1-0.7)+"px";
        this.options.titleEl.style.fontSize = "16px";
        this.options.titleEl.style.textAlign = "center";
        this.options.titleEl.style.color = "#999999";
        this.options.loadingWrap.appendChild(this.options.titleEl);
    };
    PullToRefresh.prototype.touchStart = function(ev) {
        if (this.touch.loading||!this.canPull) {
            return;
        }
        touchYDelta = '';
        this.options.imageEl.setAttribute("src",this.options.pullImage);
        this.options.container.style.webkitTransitionDuration =
        this.options.container.style.transitionDuration = '0ms';
        var touchobj = ev.changedTouches[0];
        // register first touch "y"
        firstTouchY = parseInt(touchobj.clientY);
        initialScroll = this.scrollY();
        // get load wrap height
        loadWrapH = this.options.loadingWrap.offsetHeight;
    };

    PullToRefresh.prototype.touchMove = function (ev) {
        if (this.touch.loading||!this.canPull) {
            ev.preventDefault();
            return;
        }
        var self = this;
        if (this.options.title!=undefined) {
            this.options.titleEl.textContent = this.options.textDown;
        }
        var moving = function() {
            var touchobj = ev.changedTouches[0], // reference first touch point for this event
                touchY = parseInt(touchobj.clientY);
                touchYDelta = touchY - firstTouchY;
            if ( self.scrollY() === 0 && touchYDelta > 0) {
                ev.preventDefault();
            }
            
                
            

            if ( initialScroll > 0 || self.scrollY() > 0 || self.scrollY() === 0 && touchYDelta < 0 ) {
                firstTouchY = touchY;
                return;
            }
            translateVal = -loadWrapH + touchYDelta/self.options.friction;
            if (translateVal>-WrapTop/3){
                return;
            }
            
            self.setContentTransform();
            
            if( touchYDelta >= self.options.triggerDistance ) {
                if(self.options.titleEl.textContent!=self.options.textUp){
                    self.options.titleEl.textContent = self.options.textUp;
                    self.options.imageEl.setAttribute("src",self.options.transformImage);
                }
            }else{
                if(self.options.titleEl.textContent!=self.options.textDown){
                    self.options.titleEl.textContent = self.options.textDown;
                    self.options.imageEl.setAttribute("src",self.options.pullImage);
                }
            }
        };
        this.throttle(moving(), 20);
    };
    PullToRefresh.prototype.touchEnd = function (ev) {
        var self =this;
        if (self.touch.loading||self.scrollY()!=0||!this.canPull) {
            return;
        }
        // 根据下拉高度判断是否加载
        if( touchYDelta >= self.options.triggerDistance) {
            self.touch.loading = true; //正在加载中
            ev.preventDefault();
            self.options.container.style.webkitTransform =
            self.options.container.style.transform = 'translateY('+(-WrapTop/1.5)+'px)'
            self.options.titleEl.textContent = self.options.textRefresh;
            self.options.imageEl.setAttribute("src",self.options.loadingImage);
            setTimeout(function(){
                self.options.callback('success');
            },500);
        }else{
            self.options.container.style.webkitTransform =
            self.options.container.style.transform = 'translateY('+(-WrapTop)+'px)';
            self.options.callback('cancel');
        }
        if( translateVal !== -loadWrapH ) {
            self.options.container.style.webkitTransitionDuration =
            self.options.container.style.transitionDuration = '500ms';
        }
        return;
    };

    PullToRefresh.prototype.setContentTransform = function () {
        this.options.container.style.webkitTransform =
        this.options.container.style.transform = 'translate3d(0, ' + translateVal + 'px, 0)';
    };

    PullToRefresh.prototype.cancelLoading = function (isOK) {
        var self =this;

        self.touch.loading = false;
        self.options.imageEl.setAttribute("src",this.options.pullImage);
        if(isOK){
          self.options.titleEl.textContent = self.options.textSuccess;
        }else{
          self.options.titleEl.textContent = self.options.textError;
        }
        setTimeout(function(){
            self.options.container.style.webkitTransform =
            self.options.container.style.transform = 'translateY('+(-WrapTop)+'px)';
        },500);
        return;
    };
    PullToRefresh.prototype.scrollY = function() {
        return window.pageYOffset || docElem.scrollTop;
    };
    PullToRefresh.prototype.throttle = function(fn, delay) {
        var allowSample = true;
        return function(e) {
            if (allowSample) {
                allowSample = false;
                setTimeout(function() { allowSample = true; }, delay);
                fn(e);
            }
        };
    };
    PullToRefresh.prototype.winresize = function () {
        var resize = function() {
            win = {width: window.innerWidth, height: window.innerHeight};
        };
        throttle(resize(), 10);
    };
    var extend = function (a, b) {
        for (var key in b) {
            if (b.hasOwnProperty(key)) {
                a[key] = b[key];
            }
        }
        return a;
    }
    window.mzlPullToRefresh = PullToRefresh;

})(window);